#lang racket/base
(require racket/gui/base
         racket/class
         "global.rkt"
         "mem.rkt")

;; Zoom in on Jam pixels t make them easiesr to see:
(define SCALE 5)

(define (make-screen)
  ;; A frame to show the Jam2000 screen:
  (define frame
    (new frame%
         [label "Screen"]
         [style '(no-resize-border)]
         [stretchable-width #f]
         [stretchable-height #f]))
  
  ;; A canvas to draw the Jam2000 screen:
  (define canvas
    (new canvas%
         [parent frame]
         [min-width (* SCALE WIDTH)]
         [min-height (* SCALE HEIGHT)]))

  (define (update)
    ;; Translate from the native screen format of Jam2000
    ;; to the Racket bitmap format, then draw the bitmap
    ;; to the canvas:
    (let ([bm (make-object bitmap% (* WIDTH SCALE) (* HEIGHT SCALE))]
          [s (make-bytes (* WIDTH SCALE HEIGHT SCALE 4))]
          [99->255 (lambda (v)
                     (quotient (* v 255) 99))])
      (for ([i (in-range WIDTH)])
        (for ([j (in-range HEIGHT)])
          (let ([pixel (abs (get-mem (+ SCREEN-START (+ i (* j WIDTH)))))])
            (for ([di (in-range SCALE)])
              (for ([dj (in-range SCALE)])
                (let ([p (* 4 (+ (* i SCALE) di (* (+ (* j SCALE) dj) (* SCALE WIDTH))))])
                  (bytes-set! s p 255)
                  (bytes-set! s (+ p 1) 
                              (99->255 (modulo (quotient pixel 10000) 100)))
                  (bytes-set! s (+ p 2) 
                              (99->255 (modulo (quotient pixel 100) 100)))
                  (bytes-set! s (+ p 3) 
                              (99->255 (modulo pixel 100)))))))))
      (let ([dc (make-object bitmap-dc% bm)])
        (send dc set-argb-pixels 0 0 (* WIDTH SCALE) (* HEIGHT SCALE) s)
        (send dc set-bitmap #f))
      (send (send canvas get-dc) draw-bitmap bm 0 0))
    ;; Sleep for 1/5 of a second:
    (sleep 0.2)
    ;; Recur:
    (update))

  ;; Start the display updater in its own thread:
  (thread update)

  ;; Show the Jam2000 screen:
  (send frame show #t))

;; Star the Jam2000 screen in its own GUI space:
(parameterize ([current-eventspace
                (make-eventspace)])
  (queue-callback make-screen))
